package jnpf.workflow.mapper;

import jnpf.model.workflow.dto.UserSelfDto;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface SelfApproveUserMapper {

    /**
     * 批量新增借调申请中的被借调用户信息
     * @param userIds 被借调用户列表
     * @param id 借调申请id
     * @author hlp
     */
    void addList(@Param("userIds") List<String> userIds, @Param("id") String id);

    void addUserList(@Param("userSelfDtos") List<UserSelfDto> userSelfDtos, @Param("id") String id);

    /**
     * 根据借调申请id删除关联的用户信息
     * @param id 借调申请id
     * @author hlp
     */
    void deleteById(@Param("id") String id);

    /**
     * 获取借调审批对应的借调用户列表
     * @param id 借调申请id
     * @return java.util.List<jnpf.model.workflow.dto.UserSelfDto>
     * @author hlp
     */
    List<UserSelfDto> getListById(@Param("id") String id);

    List<String> getIdListById(@Param("id") String id);

    /**
     * 绑定被借调用户
     * @param userIds 被借调用户
     * @param id 借调Id
     */
    void addUserListForOa(@Param("userIds") List<String> userIds, @Param("id") String id);
}
